<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @author Zarif Safiullin <zaph.saph@gmail.com>
 * @package ow.ow_system_plugins.base.controllers
 * @since 1.0
 */
class AJAXIM_CMP_Toolbar extends OW_Component
{

    public function render()
    {
        if ( !OW::getConfig()->getValue('ajaxim', 'is_configured') )
        {
            $this->setVisible(false);
            return;
        }

        OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('ajaxim')->getStaticJsUrl() . 'ajaxim_handlers.js');
        OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('ajaxim')->getStaticJsUrl() . 'ajaxim.js');
        OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('ajaxim')->getStaticJsUrl() . 'jquery.media.js');
        OW::getDocument()->addStyleSheet(OW::getPluginManager()->getPlugin('ajaxim')->getStaticCssUrl() . 'im.css');

        

//~~
        $node = OW::getUser()->getUserObject()->getId();
        $username = OW::getUser()->getUserObject()->getUsername();
        
        $realname = BOL_UserService::getInstance()->getDisplayName(OW::getUser()->getId());
        $avatar = BOL_AvatarService::getInstance()->getAvatarUrl(OW::getUser()->getId());
        if ( empty($avatar) )
        {
            $avatar = BOL_AvatarService::getInstance()->getDefaultAvatarUrl();
        }

        $active_list = AJAXIM_BOL_Service::getInstance()->getSessionActiveList();
        $roster_list = AJAXIM_BOL_Service::getInstance()->getSessionRosterList();

        foreach ( $roster_list as $roster )
        {
            $friendship = OW::getEventManager()->call('plugin.friends.check_friendship', array('userId' => OW::getUser()->getId(), 'friendId' => $roster->getId()));
            if ( ( empty($friendship) || $friendship->getStatus() == 'pending' ) && !isset($active_list[$roster->getId()]) )
            {
                AJAXIM_BOL_Service::getInstance()->deleteRoster(OW::getUser()->getId(), $roster->getId());
                unset($roster_list[$roster->getId()]);
            }
        }

        OW::getSession()->set('ajaxim.roster_list', $roster_list);

        $activeContact = '';
        $active_listJs = '[ ';

        if ( !empty($active_list) )
        {
            foreach ( $active_list as $contactName => $isActive )
            {
                $active_listJs .= "{$contactName}, ";
                if ( $isActive )
                {
                    $activeContact = $contactName;
                }
            }
        }

        if ( $active_listJs == '[ ' )
        {
            $active_listJs .= ' ]';
        }
        else
        {
            $active_listJs = substr($active_listJs, 0, -2) . ' ]';
        }

        $jsGenerator = UTIL_JsGenerator::newInstance();
        $jsGenerator->setVariable('activeContact', $activeContact);
        $jsGenerator->setVariable('oldTitle', OW::getDocument()->getTitle());
        $jsGenerator->setVariable('ajaxim_ping', 3000);
        //$jsGenerator->setVariable('ajaxim_ping', OW::getConfig()->getValue('ajaxim', 'ping_interval') * 1000);
        $variables = $jsGenerator->generateJs();
        OW::getDocument()->addScriptDeclaration("window.my = { node: '{$node}', realname: '{$realname}', username: '{$username}', avatar: '{$avatar}' };\n active_list = $active_listJs;\n ".$variables);

        $avatar_proto_data = array('url' => 1, 'src' => '#', 'class' => 'talk_box_avatar');
        $this->assign('avatar_proto_data', $avatar_proto_data);
        $this->assign('ajaxim_sound_url', OW::getPluginManager()->getPlugin('ajaxim')->getStaticUrl() . 'sound/receive.wav');
        
        OW::getDocument()->addScriptDeclaration('window.logMsgUrl = \'' . OW::getRouter()->urlFor('AJAXIM_CTRL_Action', 'logMsg') . '\';');
        OW::getDocument()->addScriptDeclaration('window.getLogUrl = \'' . OW::getRouter()->urlFor('AJAXIM_CTRL_Action', 'getLog') . '\';');
        OW::getDocument()->addScriptDeclaration('window.bindUrl = \'' . OW::getRouter()->urlFor('AJAXIM_CTRL_Action', 'bind') . '\';');
        OW::getDocument()->addScriptDeclaration('window.updateUserInfoUrl = \'' . OW::getRouter()->urlFor('AJAXIM_CTRL_Action', 'updateUserInfo') . '\';');

        return parent::render();
    }
}